@@ -9,7 +9,6 @@ UpSet.js Jupyter Widget
 """
 
 import typing as t
-from collections import OrderedDict
 
 from ipywidgets import DOMWidget, Layout, ValueWidget, register
 from ipywidgets.widgets.trait_types import Color
@@ -39,6 +38,7 @@ from ._model import (
     UpSetSetLike,
     UpSetSetUnion,
     UpSetFontSizes,
+    UpSetAttribute,
 )
 from ._generate import (
     generate_unions,
@@ -98,6 +98,7 @@ def _to_set_list(arr: t.List[UpSetSet], model: "UpSetJSWidget"):
         dict(
             type=str(s.set_type),
             name=s.name,
+            color=s.color,
             cardinality=s.cardinality,
             elems=compress_index_array(model.elem_to_index[e] for e in s.elems),
         )
@@ -110,6 +111,7 @@ def _to_combination_list(arr: t.List[UpSetSetCombination], model: "UpSetJSWidget
         dict(
             type=str(s.set_type),
             name=s.name,
+            color=s.color,
             cardinality=s.cardinality,
             degree=s.degree,
             set_names=[c.name for c in s.sets],
@@ -133,9 +135,8 @@ def _to_query_list(arr: t.List[UpSetQuery], model: "UpSetJSWidget"):
     return [_to_query(q) for q in arr]
 
 
-@register
-class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
-    """UpSet.js Widget
+class UpSetJSBaseWidget(ValueWidget, DOMWidget, t.Generic[T]):
+    """UpSet.js Base Widget
     """
 
     _model_name = Unicode("UpSetModel").tag(sync=True)
@@ -145,33 +146,20 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
     _view_module = Unicode(MODULE_NAME).tag(sync=True)
     _view_module_version = Unicode(MODULE_VERSION).tag(sync=True)
 
-    """
-    interactivity mode of the widget whether the plot is static, reacts on hover or click events
-    """
     mode: str = Enum(
         ("hover", "click", "static", "contextMenu"), default_value="hover"
     ).tag(sync=True)
     """
-    padding within the svg
+    interactivity mode of the widget whether the plot is static, reacts on hover or click events
     """
     padding: float = Float(None, allow_none=True).tag(sync=True)
     """
-    padding argument for scaleBand (0..1)
+    padding within the svg
     """
-    bar_padding: float = Float(None, allow_none=True).tag(sync=True)
-    dot_padding: float = Float(None, allow_none=True).tag(sync=True)
-    width_ratios: t.Tuple[float, float, float] = Tuple(
-        Float(), Float(), Float(), default_value=(0.25, 0.1, 0.65)
-    ).tag(sync=True)
-    height_ratios: t.Tuple[float, float] = Tuple(
-        Float(), Float(), default_value=(0.6, 0.4)
-    ).tag(sync=True)
 
     elems: t.List[T] = List(default_value=[]).tag(sync=True)
     elem_to_index: t.Dict[T, int] = {}
 
-    attrs: t.Dict[str, t.List[float]] = Dict().tag(sync=True)
-
     sets: t.List[UpSetSet[T]] = List(Instance(UpSetSet), default_value=[],).tag(
         sync=True, to_json=_to_set_list
     )
@@ -190,17 +178,15 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
     )
 
     theme: str = Enum(("light", "dark", "vega"), default_value="light").tag(sync=True)
-    selection_color: str = Color(None, allow_none=True).tag(sync=True)
+    selection_color: str = Union(
+        (Color(None, allow_none=True), Unicode(None, allow_none=True))
+    ).tag(sync=True)
     has_selection_color: str = Color(None, allow_none=True).tag(sync=True)
-    alternating_background_color: str = Color(None, allow_none=True).tag(sync=True)
+    opacity: float = Float(None, allow_none=True).tag(sync=True)
+    has_selection_opacity: float = Float(None, allow_none=True).tag(sync=True)
+
     color: str = Color(None, allow_none=True).tag(sync=True)
     text_color: str = Color(None, allow_none=True).tag(sync=True)
-    hover_hint_color: str = Color(None, allow_none=True).tag(sync=True)
-    not_member_color: str = Color(None, allow_none=True).tag(sync=True)
-
-    bar_label_offset: float = Float(None, allow_none=True).tag(sync=True)
-    set_name_axis_offset: float = Float(None, allow_none=True).tag(sync=True)
-    combination_name_axis_offset: float = Float(None, allow_none=True).tag(sync=True)
 
     query_legend: bool = Bool(None, allow_none=True).tag(sync=True)
     export_buttons: bool = Bool(None, allow_none=True).tag(sync=True)
@@ -208,11 +194,7 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
     font_sizes: UpSetFontSizes = Instance(UpSetFontSizes).tag(
         sync=True, to_json=lambda v, _: v.to_json()
     )
-    numeric_scale: str = Enum(("linear", "log"), default_value="linear").tag(sync=True)
-    band_scale: str = Enum(("band"), default_value="band").tag(sync=True)
 
-    set_name: str = Unicode(None, allow_none=True).tag(sync=True)
-    combination_name: str = Unicode(None, allow_none=True).tag(sync=True)
     title: str = Unicode(None, allow_none=True).tag(sync=True)
     description: str = Unicode(None, allow_none=True).tag(sync=True)
 
@@ -220,20 +202,11 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
         super().__init__(**kwargs)
         self.observe(self._sync_value, "value")
 
-    def copy(self) -> "UpSetJSWidget":
-        """
-        returns a copy of itself
-        """
-        clone = UpSetJSWidget[T]()  # pylint: disable=unsubscriptable-object
-
+    def _base_copy(self, clone: "UpSetJSBaseWidget"):
         clone.title = self.title
         clone.description = self.description
         clone.mode = self.mode
         clone.padding = self.padding
-        clone.bar_padding = self.bar_padding
-        clone.dot_padding = self.dot_padding
-        clone.width_ratios = self.width_ratios
-        clone.height_ratios = self.height_ratios
         clone.elems = list(self.elems)
         clone.elem_to_index = self.elem_to_index.copy()
         clone.sets = list(self.sets)
@@ -245,28 +218,16 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
 
         clone.theme = self.theme
         clone.selection_color = self.selection_color
-        clone.alternating_background_color = self.alternating_background_color
         clone.color = self.color
         clone.has_selection_color = self.has_selection_color
+        clone.opacity = self.opacity
+        clone.has_selection_opacity = self.has_selection_opacity
         clone.text_color = self.text_color
-        clone.hover_hint_color = self.hover_hint_color
-        clone.not_member_color = self.not_member_color
-
-        clone.bar_label_offset = self.bar_label_offset
-        clone.set_name_axis_offset = self.set_name_axis_offset
-        clone.combination_name_axis_offset = self.combination_name_axis_offset
 
         clone.query_legend = self.query_legend
         clone.export_buttons = self.export_buttons
         clone.font_family = self.font_family
         clone.font_sizes = self.font_sizes.copy()
-        clone.numeric_scale = self.numeric_scale
-        clone.band_scale = self.band_scale
-
-        clone.set_name = self.set_name
-        clone.combination_name = self.combination_name
-
-        return clone
 
     def _sync_value(self, evt: t.Any):
         self._selection = self._value_to_selection(evt["new"])
@@ -284,10 +245,10 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
         if isinstance(value, list):
             return frozenset([self.elems[i] for i in value])
         if isinstance(value, dict):
-            typee = value.get("type", "set")
+            set_type = value.get("type", "set")
             name = value["name"]
 
-            if typee == "set":
+            if set_type == "set":
                 found_set = next((s for s in self.sets if s.name == name), None)
                 if found_set:
                     return found_set
@@ -296,7 +257,7 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
                     (
                         s
                         for s in self.combinations
-                        if s.name == name and str(s.set_type) == typee
+                        if s.name == name and str(s.set_type) == set_type
                     ),
                     None,
                 )
@@ -307,13 +268,13 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
             elems = frozenset(self.elems[i] for i in value.get("elems", []))
             set_by_name = {s.name: s for s in self.sets}
             sets = frozenset(set_by_name[n] for n in value.get("set_names", []))
-            if typee == "set":
+            if set_type == "set":
                 return UpSetSet[T](name, elems)
-            if typee == "intersection":
+            if set_type == "intersection":
                 return UpSetSetIntersection[T](name, elems, sets)
-            if typee == "distinctIntersection":
+            if set_type == "distinctIntersection":
                 return UpSetSetDistinctIntersection[T](name, elems, sets)
-            if typee == "union":
+            if set_type == "union":
                 return UpSetSetUnion[T](name, elems, sets)
             return UpSetSetComposite[T](name, elems, sets)
         return None
@@ -421,12 +382,13 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
     def _default_font_sizes(self):  # pylint: disable=no-self-use
         return UpSetFontSizes()
 
-    def from_dict(
+    def _from_dict(
         self,
         sets: t.Dict[str, t.Sequence[T]],
         order_by: str = "cardinality",
         limit: t.Optional[int] = None,
-    ) -> "UpSetJSWidget":
+        colors: t.Mapping[str, str] = None,
+    ) -> t.List[UpSetSet[T]]:
         """
         generates the list of sets from a dict
         """
@@ -435,36 +397,42 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
             elems.update(set_elems)
         self.elems = sorted(elems)
         self.elem_to_index = {e: i for i, e in enumerate(self.elems)}
+        color_lookup = colors or {}
 
         base_sets: t.List[UpSetSet[T]] = [
-            UpSetSet[T](name=k, elems=frozenset(v)) for k, v in sets.items()
+            UpSetSet[T](name=k, elems=frozenset(v), color=color_lookup.get(k))
+            for k, v in sets.items()
         ]
         self.clear_queries()
         self.selection = None
         self.sets = _sort_sets(base_sets, order_by, limit)
-        self.attrs = OrderedDict()
-        return self.generate_intersections(order_by=order_by)
 
-    def from_dataframe(
+        return base_sets if limit is None else self.sets
+
+    def _from_dataframe(
         self,
         data_frame: t.Any,
         attributes: t.Union[t.Sequence[str], t.Any, None] = None,
         order_by: str = "cardinality",
         limit: t.Optional[int] = None,
-    ):
+        colors: t.Mapping[str, str] = None,
+    ) -> t.List[UpSetSet[T]]:
         """
         generates the list of sets from a dataframe
         """
         self.elems = sorted(data_frame.index)
         self.elem_to_index = {e: i for i, e in enumerate(self.elems)}
+        color_lookup = colors or {}
 
         def to_set(name: str, series):
             elems = series[series.astype(bool)].index
-            return UpSetSet[T](name=name, elems=frozenset(elems))
+            return UpSetSet[T](
+                name=name, elems=frozenset(elems), color=color_lookup.get(name)
+            )
 
         attribute_columns = attributes if isinstance(attributes, (list, tuple)) else []
 
-        base_sets = [
+        base_sets: t.List[UpSetSet[T]] = [
             to_set(name, series)
             for name, series in data_frame.items()
             if name not in attribute_columns
@@ -472,6 +440,111 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
         self.clear_queries()
         self.selection = None
         self.sets = _sort_sets(base_sets, order_by, limit)
+        return base_sets if limit is None else self.sets
+
+
+@register
+class UpSetJSWidget(UpSetJSBaseWidget, t.Generic[T]):
+    """UpSet.js Widget
+    """
+
+    _render_mode = Unicode("upset").tag(sync=True)
+
+    bar_padding: float = Float(None, allow_none=True).tag(sync=True)
+    """
+    padding argument for scaleBand (0..1)
+    """
+    dot_padding: float = Float(None, allow_none=True).tag(sync=True)
+    width_ratios: t.Tuple[float, float, float] = Tuple(
+        Float(), Float(), Float(), default_value=(0.25, 0.1, 0.65)
+    ).tag(sync=True)
+    height_ratios: t.Tuple[float, float] = Tuple(
+        Float(), Float(), default_value=(0.6, 0.4)
+    ).tag(sync=True)
+
+    attrs: t.List[UpSetAttribute[T]] = List(
+        Instance(UpSetAttribute), default_value=[]
+    ).tag(sync=True, to_json=lambda v, _: [vi.to_json() for vi in v])
+
+    alternating_background_color: str = Color(None, allow_none=True).tag(sync=True)
+    hover_hint_color: str = Color(None, allow_none=True).tag(sync=True)
+    not_member_color: str = Color(None, allow_none=True).tag(sync=True)
+
+    bar_label_offset: float = Float(None, allow_none=True).tag(sync=True)
+    set_name_axis_offset: float = Float(None, allow_none=True).tag(sync=True)
+    combination_name_axis_offset: float = Float(None, allow_none=True).tag(sync=True)
+
+    numeric_scale: str = Enum(("linear", "log"), default_value="linear").tag(sync=True)
+    band_scale: str = Enum(("band"), default_value="band").tag(sync=True)
+
+    set_name: str = Unicode(None, allow_none=True).tag(sync=True)
+    combination_name: str = Unicode(None, allow_none=True).tag(sync=True)
+
+    def copy(self) -> "UpSetJSWidget":
+        """
+        returns a copy of itself
+        """
+        clone = UpSetJSWidget[T]()  # pylint: disable=unsubscriptable-object
+        self._base_copy(clone)
+        clone.bar_padding = self.bar_padding
+        clone.dot_padding = self.dot_padding
+        clone.width_ratios = self.width_ratios
+        clone.height_ratios = self.height_ratios
+
+        clone.alternating_background_color = self.alternating_background_color
+        clone.hover_hint_color = self.hover_hint_color
+        clone.not_member_color = self.not_member_color
+
+        clone.bar_label_offset = self.bar_label_offset
+        clone.set_name_axis_offset = self.set_name_axis_offset
+        clone.combination_name_axis_offset = self.combination_name_axis_offset
+
+        clone.numeric_scale = self.numeric_scale
+        clone.band_scale = self.band_scale
+
+        clone.set_name = self.set_name
+        clone.combination_name = self.combination_name
+
+        return clone
+
+    def from_dict(
+        self,
+        sets: t.Dict[str, t.Sequence[T]],
+        order_by: str = "cardinality",
+        limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
+    ) -> "UpSetJSWidget":
+        """
+        generates the list of sets from a dict
+        """
+        base_sets = super()._from_dict(sets, order_by, limit, colors)
+        self.attrs = []
+        return self._generate_intersections(base_sets, order_by, colors)
+
+    def from_dataframe(
+        self,
+        data_frame: t.Any,
+        attributes: t.Union[t.Sequence[str], t.Any, None] = None,
+        order_by: str = "cardinality",
+        limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
+    ):
+        """
+        generates the list of sets from a dataframe
+        """
+        base_sets = super()._from_dataframe(
+            data_frame, attributes, order_by, limit, colors
+        )
+
+        def as_attr(name: str, values: t.List):
+            attr_type = (
+                "categorical" if not values or isinstance(values[0], str) else "number"
+            )
+            domain = (min(values), max(values)) if attr_type == "number" else None
+            categories = sorted(set(values)) if attr_type == "categorical" else None
+            return UpSetAttribute[T](
+                attr_type, name, values, domain=domain, categories=categories
+            )
 
         if attributes is not None:
             attribute_df = (
@@ -479,13 +552,31 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
                 if isinstance(attributes, (list, tuple))
                 else attributes
             )
-            self.attrs = OrderedDict(
-                [(name, series.tolist()) for name, series in attribute_df.items()]
-            )
+            self.attrs = [
+                as_attr(name, series.tolist()) for name, series in attribute_df.items()
+            ]
         else:
-            self.attrs = OrderedDict()
+            self.attrs = []
 
-        return self.generate_intersections(order_by=order_by)
+        return self._generate_intersections(base_sets, order_by, colors)
+
+    def _generate_intersections(
+        self,
+        base_sets: t.List[UpSetSet[T]],
+        order_by: t.Union[str, t.Sequence[str]] = "cardinality",
+        colors: t.Mapping[str, str] = None,
+    ):
+        """
+        customize the generation of the sets
+        """
+        set_intersections = generate_intersections(
+            base_sets, 0, None, False, self.elems, colors=colors
+        )
+
+        self.combinations = _sort_combinations(
+            set_intersections, self.sets, order_by, None
+        )
+        return self
 
     def generate_intersections(
         self,
@@ -494,12 +585,13 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
         empty: bool = False,
         order_by: t.Union[str, t.Sequence[str]] = "cardinality",
         limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
     ):
         """
         customize the generation of the sets
         """
         set_intersections = generate_intersections(
-            self.sets, min_degree, max_degree, empty, self.elems
+            self.sets, min_degree, max_degree, empty, self.elems, colors=colors
         )
 
         self.combinations = _sort_combinations(
@@ -514,12 +606,13 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
         empty: bool = False,
         order_by: t.Union[str, t.Sequence[str]] = "cardinality",
         limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
     ):
         """
         customize the generation of the sets
         """
         set_intersections = generate_distinct_intersections(
-            self.sets, min_degree, max_degree, empty, self.elems
+            self.sets, min_degree, max_degree, empty, self.elems, colors=colors,
         )
 
         self.combinations = _sort_combinations(
@@ -534,13 +627,126 @@ class UpSetJSWidget(ValueWidget, DOMWidget, t.Generic[T]):
         empty: bool = False,
         order_by: t.Union[str, t.Sequence[str]] = "cardinality",
         limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
     ):
         """
         customize the generation of the sets
         """
         set_unions = generate_unions(
-            self.sets, min_degree, max_degree, empty, self.elems
+            self.sets, min_degree, max_degree, empty, self.elems, colors=colors,
         )
 
         self.combinations = _sort_combinations(set_unions, self.sets, order_by, limit)
         return self
+
+    def clear_attributes(self):
+        """
+        deletes the list of attributes
+        """
+        self.attrs = []
+
+    def append_numeric_attribute(
+        self,
+        name: str,
+        values: t.List[float],
+        min_value: t.Optional[float] = None,
+        max_value: t.Optional[float] = None,
+    ) -> "UpSetJSWidget":
+        """
+        adds another numerical UpSetAttribute to be visualized
+        """
+        domain = (
+            min_value if min_value is not None else min(values),
+            max_value if max_value is not None else max(values),
+        )
+        self.attrs = self.attrs + [UpSetAttribute[T]("number", name, values, domain)]
+        return self
+
+    def append_categorical_attribute(
+        self,
+        name: str,
+        values: t.List[str],
+        categories: t.Optional[t.List[t.Union[str, t.Dict]]] = None,
+    ) -> "UpSetJSWidget":
+        """
+        adds another categorical UpSetAttribute to be visualized
+        """
+        cats = categories if categories is not None else sorted(set(values))
+        self.attrs = self.attrs + [
+            UpSetAttribute[T]("categorical", name, values, categories=cats)
+        ]
+        return self
+
+
+@register
+class UpSetJSVennDiagramWidget(UpSetJSBaseWidget, t.Generic[T]):
+    """UpSet.js Venn Diagram Widget
+    """
+
+    _render_mode = Unicode("venn").tag(sync=True)
+
+    value_text_color: str = Color(None, allow_none=True).tag(sync=True)
+    stroke_color: str = Color(None, allow_none=True).tag(sync=True)
+
+    def copy(self) -> "UpSetJSVennDiagramWidget":
+        """
+        returns a copy of itself
+        """
+        clone = UpSetJSVennDiagramWidget[T]()  # pylint: disable=unsubscriptable-object
+        self._base_copy(clone)
+        clone.value_text_color = self.value_text_color
+        clone.stroke_color = self.stroke_color
+
+        return clone
+
+    def from_dict(
+        self,
+        sets: t.Dict[str, t.Sequence[T]],
+        order_by: str = "cardinality",
+        limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
+    ) -> "UpSetJSVennDiagramWidget":
+        """
+        generates the list of sets from a dict
+        """
+        base_sets = super()._from_dict(sets, order_by, limit, colors)
+        return self._generate_distinct_intersections(base_sets, colors=colors)
+
+    def from_dataframe(
+        self,
+        data_frame: t.Any,
+        attributes: t.Union[t.Sequence[str], t.Any, None] = None,
+        order_by: str = "cardinality",
+        limit: t.Optional[int] = None,
+        colors: t.Mapping[str, str] = None,
+    ):
+        """
+        generates the list of sets from a dataframe
+        """
+        base_sets = super()._from_dataframe(
+            data_frame, attributes, order_by, limit, colors
+        )
+
+        return self._generate_distinct_intersections(base_sets, colors=colors)
+
+    def _generate_distinct_intersections(
+        self, base_sets: t.List[UpSetSet[T]], colors: t.Mapping[str, str] = None,
+    ):
+        set_intersections = generate_distinct_intersections(
+            base_sets, 1, None, True, self.elems, colors=colors
+        )
+
+        self.combinations = _sort_combinations(
+            set_intersections, self.sets, ["degree", "group"]
+        )
+        return self
+
+
+@register
+class UpSetJSEulerDiagramWidget(UpSetJSVennDiagramWidget, t.Generic[T]):
+    """UpSet.js Euler Diagram Widget
+    """
+
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self._render_mode = "euler"
